from crypticorn.common.errors import *
from crypticorn.common.scopes import *
from crypticorn.common.urls import *
from crypticorn.common.decorators import *
from crypticorn.common.mixins import *
from crypticorn.common.auth import *
from crypticorn.common.enums import *
from crypticorn.common.utils import *
from crypticorn.common.exceptions import *



# coding: utf-8

"""
Marketcap Service API

No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

The version of the OpenAPI document: 1.0.0
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501

import warnings
from pydantic import validate_call, Field, StrictFloat, StrictStr, StrictInt
from typing import Any, Dict, List, Optional, Tuple, Union
from typing_extensions import Annotated

from pydantic import Field, StrictInt, StrictStr
from typing import Any, Dict, List, Optional
from typing_extensions import Annotated
from crypticorn.metrics.client.models.internal_exchange import InternalExchange
from crypticorn.metrics.client.models.market_type import MarketType
from crypticorn.metrics.client.models.time_interval import TimeInterval
from crypticorn.metrics.client.models.trading_status import TradingStatus

from crypticorn.metrics.client.api_client import ApiClient, RequestSerialized
from crypticorn.metrics.client.api_response import ApiResponse
from crypticorn.metrics.client.rest import RESTResponseType


class ExchangesApi:
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, api_client=None) -> None:
        if api_client is None:
            api_client = ApiClient.get_default()
        self.api_client = api_client

    @validate_call
    async def get_available_exchanges(
        self,
        market: Annotated[
            MarketType, Field(description="Market type (spot or futures)")
        ],
        symbol: Annotated[
            StrictStr, Field(description="Symbol to fetch available exchanges for")
        ],
        interval: Annotated[
            Optional[TimeInterval],
            Field(description="Interval for which to fetch available exchanges"),
        ] = None,
        start_timestamp: Annotated[
            Optional[StrictInt],
            Field(
                description="Start timestamp for which to fetch available exchanges (defaults to previous 7 day's closing)"
            ),
        ] = None,
        end_timestamp: Annotated[
            Optional[StrictInt],
            Field(description="End timestamp for which to fetch available exchanges"),
        ] = None,
        quote_currency: Annotated[
            Optional[StrictStr],
            Field(
                description="Quote currency for which to fetch available exchanges (Use quote currencies endpoint to get available quote currencies)"
            ),
        ] = None,
        status: Annotated[
            Optional[TradingStatus],
            Field(
                description="Trading pair status for which to fetch available exchanges"
            ),
        ] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> List[Dict[str, object]]:
        """Get Available Exchanges

        Get available exchanges for a symbol with various filtering options.

        :param market: Market type (spot or futures) (required)
        :type market: MarketType
        :param symbol: Symbol to fetch available exchanges for (required)
        :type symbol: str
        :param interval: Interval for which to fetch available exchanges
        :type interval: TimeInterval
        :param start_timestamp: Start timestamp for which to fetch available exchanges (defaults to previous 7 day's closing)
        :type start_timestamp: int
        :param end_timestamp: End timestamp for which to fetch available exchanges
        :type end_timestamp: int
        :param quote_currency: Quote currency for which to fetch available exchanges (Use quote currencies endpoint to get available quote currencies)
        :type quote_currency: str
        :param status: Trading pair status for which to fetch available exchanges
        :type status: TradingStatus
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._get_available_exchanges_serialize(
            market=market,
            symbol=symbol,
            interval=interval,
            start_timestamp=start_timestamp,
            end_timestamp=end_timestamp,
            quote_currency=quote_currency,
            status=status,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "List[Dict[str, object]]",
        }
        response_data = await self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        await response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data

    @validate_call
    async def get_available_exchanges_with_http_info(
        self,
        market: Annotated[
            MarketType, Field(description="Market type (spot or futures)")
        ],
        symbol: Annotated[
            StrictStr, Field(description="Symbol to fetch available exchanges for")
        ],
        interval: Annotated[
            Optional[TimeInterval],
            Field(description="Interval for which to fetch available exchanges"),
        ] = None,
        start_timestamp: Annotated[
            Optional[StrictInt],
            Field(
                description="Start timestamp for which to fetch available exchanges (defaults to previous 7 day's closing)"
            ),
        ] = None,
        end_timestamp: Annotated[
            Optional[StrictInt],
            Field(description="End timestamp for which to fetch available exchanges"),
        ] = None,
        quote_currency: Annotated[
            Optional[StrictStr],
            Field(
                description="Quote currency for which to fetch available exchanges (Use quote currencies endpoint to get available quote currencies)"
            ),
        ] = None,
        status: Annotated[
            Optional[TradingStatus],
            Field(
                description="Trading pair status for which to fetch available exchanges"
            ),
        ] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[List[Dict[str, object]]]:
        """Get Available Exchanges

        Get available exchanges for a symbol with various filtering options.

        :param market: Market type (spot or futures) (required)
        :type market: MarketType
        :param symbol: Symbol to fetch available exchanges for (required)
        :type symbol: str
        :param interval: Interval for which to fetch available exchanges
        :type interval: TimeInterval
        :param start_timestamp: Start timestamp for which to fetch available exchanges (defaults to previous 7 day's closing)
        :type start_timestamp: int
        :param end_timestamp: End timestamp for which to fetch available exchanges
        :type end_timestamp: int
        :param quote_currency: Quote currency for which to fetch available exchanges (Use quote currencies endpoint to get available quote currencies)
        :type quote_currency: str
        :param status: Trading pair status for which to fetch available exchanges
        :type status: TradingStatus
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._get_available_exchanges_serialize(
            market=market,
            symbol=symbol,
            interval=interval,
            start_timestamp=start_timestamp,
            end_timestamp=end_timestamp,
            quote_currency=quote_currency,
            status=status,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "List[Dict[str, object]]",
        }
        response_data = await self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        await response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )

    @validate_call
    async def get_available_exchanges_without_preload_content(
        self,
        market: Annotated[
            MarketType, Field(description="Market type (spot or futures)")
        ],
        symbol: Annotated[
            StrictStr, Field(description="Symbol to fetch available exchanges for")
        ],
        interval: Annotated[
            Optional[TimeInterval],
            Field(description="Interval for which to fetch available exchanges"),
        ] = None,
        start_timestamp: Annotated[
            Optional[StrictInt],
            Field(
                description="Start timestamp for which to fetch available exchanges (defaults to previous 7 day's closing)"
            ),
        ] = None,
        end_timestamp: Annotated[
            Optional[StrictInt],
            Field(description="End timestamp for which to fetch available exchanges"),
        ] = None,
        quote_currency: Annotated[
            Optional[StrictStr],
            Field(
                description="Quote currency for which to fetch available exchanges (Use quote currencies endpoint to get available quote currencies)"
            ),
        ] = None,
        status: Annotated[
            Optional[TradingStatus],
            Field(
                description="Trading pair status for which to fetch available exchanges"
            ),
        ] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Get Available Exchanges

        Get available exchanges for a symbol with various filtering options.

        :param market: Market type (spot or futures) (required)
        :type market: MarketType
        :param symbol: Symbol to fetch available exchanges for (required)
        :type symbol: str
        :param interval: Interval for which to fetch available exchanges
        :type interval: TimeInterval
        :param start_timestamp: Start timestamp for which to fetch available exchanges (defaults to previous 7 day's closing)
        :type start_timestamp: int
        :param end_timestamp: End timestamp for which to fetch available exchanges
        :type end_timestamp: int
        :param quote_currency: Quote currency for which to fetch available exchanges (Use quote currencies endpoint to get available quote currencies)
        :type quote_currency: str
        :param status: Trading pair status for which to fetch available exchanges
        :type status: TradingStatus
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._get_available_exchanges_serialize(
            market=market,
            symbol=symbol,
            interval=interval,
            start_timestamp=start_timestamp,
            end_timestamp=end_timestamp,
            quote_currency=quote_currency,
            status=status,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "List[Dict[str, object]]",
        }
        response_data = await self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        return response_data.response

    def _get_available_exchanges_serialize(
        self,
        market,
        symbol,
        interval,
        start_timestamp,
        end_timestamp,
        quote_currency,
        status,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {}

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if market is not None:
            _path_params["market"] = market.value
        if symbol is not None:
            _path_params["symbol"] = symbol
        # process the query parameters
        if interval is not None:

            _query_params.append(("interval", interval.value))

        if start_timestamp is not None:

            _query_params.append(("start_timestamp", start_timestamp))

        if end_timestamp is not None:

            _query_params.append(("end_timestamp", end_timestamp))

        if quote_currency is not None:

            _query_params.append(("quote_currency", quote_currency))

        if status is not None:

            _query_params.append(("status", status.value))

        # process the header parameters
        # process the form parameters
        # process the body parameter

        # set the HTTP header `Accept`
        if "Accept" not in _header_params:
            _header_params["Accept"] = self.api_client.select_header_accept(
                ["application/json"]
            )

        # authentication setting
        _auth_settings: List[str] = ["APIKeyHeader", "HTTPBearer"]

        return self.api_client.param_serialize(
            method="GET",
            resource_path="/available_exchanges/{market}/{symbol}",
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth,
        )

    @validate_call
    async def get_available_exchanges_for_market(
        self,
        market: Annotated[
            MarketType, Field(description="Market type (spot or futures)")
        ],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> List[str]:
        """Get Exchange List

        Get list of exchanges for a market.

        :param market: Market type (spot or futures) (required)
        :type market: MarketType
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._get_available_exchanges_for_market_serialize(
            market=market,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "List[str]",
        }
        response_data = await self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        await response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data

    @validate_call
    async def get_available_exchanges_for_market_with_http_info(
        self,
        market: Annotated[
            MarketType, Field(description="Market type (spot or futures)")
        ],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[List[str]]:
        """Get Exchange List

        Get list of exchanges for a market.

        :param market: Market type (spot or futures) (required)
        :type market: MarketType
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._get_available_exchanges_for_market_serialize(
            market=market,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "List[str]",
        }
        response_data = await self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        await response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )

    @validate_call
    async def get_available_exchanges_for_market_without_preload_content(
        self,
        market: Annotated[
            MarketType, Field(description="Market type (spot or futures)")
        ],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Get Exchange List

        Get list of exchanges for a market.

        :param market: Market type (spot or futures) (required)
        :type market: MarketType
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._get_available_exchanges_for_market_serialize(
            market=market,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "List[str]",
        }
        response_data = await self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        return response_data.response

    def _get_available_exchanges_for_market_serialize(
        self,
        market,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {}

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if market is not None:
            _path_params["market"] = market.value
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter

        # set the HTTP header `Accept`
        if "Accept" not in _header_params:
            _header_params["Accept"] = self.api_client.select_header_accept(
                ["application/json"]
            )

        # authentication setting
        _auth_settings: List[str] = ["APIKeyHeader", "HTTPBearer"]

        return self.api_client.param_serialize(
            method="GET",
            resource_path="/exchange_list/{market}",
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth,
        )

    @validate_call
    async def get_exchange_mappings(
        self,
        market: Annotated[
            MarketType, Field(description="Market type (spot or futures)")
        ],
        exchange: Annotated[
            Optional[InternalExchange],
            Field(description="Exchange name for which to fetch exchange mappings"),
        ] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> List[str]:
        """Get Exchange Mappings

        Get exchange mappings for a market with optional exchange name filter.

        :param market: Market type (spot or futures) (required)
        :type market: MarketType
        :param exchange: Exchange name for which to fetch exchange mappings
        :type exchange: InternalExchange
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._get_exchange_mappings_serialize(
            market=market,
            exchange=exchange,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "List[str]",
        }
        response_data = await self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        await response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data

    @validate_call
    async def get_exchange_mappings_with_http_info(
        self,
        market: Annotated[
            MarketType, Field(description="Market type (spot or futures)")
        ],
        exchange: Annotated[
            Optional[InternalExchange],
            Field(description="Exchange name for which to fetch exchange mappings"),
        ] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[List[str]]:
        """Get Exchange Mappings

        Get exchange mappings for a market with optional exchange name filter.

        :param market: Market type (spot or futures) (required)
        :type market: MarketType
        :param exchange: Exchange name for which to fetch exchange mappings
        :type exchange: InternalExchange
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._get_exchange_mappings_serialize(
            market=market,
            exchange=exchange,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "List[str]",
        }
        response_data = await self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        await response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )

    @validate_call
    async def get_exchange_mappings_without_preload_content(
        self,
        market: Annotated[
            MarketType, Field(description="Market type (spot or futures)")
        ],
        exchange: Annotated[
            Optional[InternalExchange],
            Field(description="Exchange name for which to fetch exchange mappings"),
        ] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)], Annotated[StrictFloat, Field(gt=0)]
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Get Exchange Mappings

        Get exchange mappings for a market with optional exchange name filter.

        :param market: Market type (spot or futures) (required)
        :type market: MarketType
        :param exchange: Exchange name for which to fetch exchange mappings
        :type exchange: InternalExchange
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._get_exchange_mappings_serialize(
            market=market,
            exchange=exchange,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            "200": "List[str]",
        }
        response_data = await self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        return response_data.response

    def _get_exchange_mappings_serialize(
        self,
        market,
        exchange,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {}

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if market is not None:
            _path_params["market"] = market.value
        # process the query parameters
        if exchange is not None:

            _query_params.append(("exchange", exchange.value))

        # process the header parameters
        # process the form parameters
        # process the body parameter

        # set the HTTP header `Accept`
        if "Accept" not in _header_params:
            _header_params["Accept"] = self.api_client.select_header_accept(
                ["application/json"]
            )

        # authentication setting
        _auth_settings: List[str] = ["APIKeyHeader", "HTTPBearer"]

        return self.api_client.param_serialize(
            method="GET",
            resource_path="/exchange_mappings/{market}",
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth,
        )



class ValidateEnumMixin:
    """
    Mixin for validating enum values manually.

    ⚠️ Note:
    This does NOT enforce validation automatically on enum creation.
    It's up to the developer to call `Class.validate(value)` where needed.

    Usage:
        >>> class Color(ValidateEnumMixin, StrEnum):
        >>>     RED = "red"
        >>>     GREEN = "green"

        >>> Color.validate("red")     # True
        >>> Color.validate("yellow")  # False

    Order of inheritance matters — the mixin must come first.
    """

    @classmethod
    def validate(cls, value) -> bool:
        """Validate if a value is in the enum. True if so, False otherwise."""
        try:
            cls(value)
            return True
        except ValueError:
            return False

class ExcludeEnumMixin:
    """Mixin to exclude enum from OpenAPI schema. We use this to avoid duplicating enums when generating client code from the openapi spec."""

    @classmethod
    def __get_pydantic_json_schema__(cls, core_schema, handler):
        schema = handler(core_schema)
        schema.pop("enum", None)
        return schema



# coding: utf-8

"""
Payment API

No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

The version of the OpenAPI document: 0.1.0
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field, StrictFloat, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Union
from typing import Optional, Set
from typing_extensions import Self


class EstimatePriceRes(BaseModel):
    """
    Response for the estimate price calculation.  Contains the source currency and amount, target currency, and the estimated conversion amount. https://documenter.getpostman.com/view/7907941/2s93JusNJt#3c86a16e-94ad-4230-a470-4e833766a4c7
    """  # noqa: E501

    currency_from: StrictStr = Field(description="Source currency")
    amount_from: Union[StrictFloat, StrictInt] = Field(
        description="Original amount in source currency"
    )
    currency_to: StrictStr = Field(description="Target cryptocurrency")
    estimated_amount: Union[StrictFloat, StrictInt] = Field(
        description="Estimated amount in target cryptocurrency"
    )
    __properties: ClassVar[List[str]] = [
        "currency_from",
        "amount_from",
        "currency_to",
        "estimated_amount",
    ]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of EstimatePriceRes from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of EstimatePriceRes from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate(
            {
                "currency_from": obj.get("currency_from"),
                "amount_from": obj.get("amount_from"),
                "currency_to": obj.get("currency_to"),
                "estimated_amount": obj.get("estimated_amount"),
            }
        )
        return _obj



from typing import Optional, Dict, Any
from pydantic import BaseModel, Field
from fastapi import HTTPException as FastAPIHTTPException, Request, FastAPI
from fastapi.exceptions import RequestValidationError, ResponseValidationError
from fastapi.responses import JSONResponse
from crypticorn.common import ApiError, ApiErrorIdentifier, ApiErrorType, ApiErrorLevel


class ExceptionDetail(BaseModel):
    """This is the detail of the exception. It is used to enrich the exception with additional information by unwrapping the ApiError into its components."""

    message: Optional[str] = Field(None, description="An additional error message")
    code: ApiErrorIdentifier = Field(..., description="The unique error code")
    type: ApiErrorType = Field(..., description="The type of error")
    level: ApiErrorLevel = Field(..., description="The level of the error")
    status_code: int = Field(..., description="The HTTP status code")
    details: Any = Field(None, description="Additional details about the error")


class ExceptionContent(BaseModel):
    """This is the detail of the exception. Pass an ApiError to the constructor and an optional human readable message."""

    error: ApiError = Field(..., description="The unique error code")
    message: Optional[str] = Field(None, description="An additional error message")
    details: Any = Field(None, description="Additional details about the error")

    def enrich(self) -> ExceptionDetail:
        return ExceptionDetail(
            message=self.message,
            code=self.error.identifier,
            type=self.error.type,
            level=self.error.level,
            status_code=self.error.status_code,
            details=self.details,
        )


class HTTPException(FastAPIHTTPException):
    """A custom HTTP exception wrapper around FastAPI's HTTPException.
    It allows for a more structured way to handle errors, with a message and an error code. The status code is being derived from the detail's error.
        The ApiError class is the source of truth for everything. If the error is not yet implemented, there are fallbacks to avoid errors while testing.
    """

    def __init__(
        self,
        content: ExceptionContent,
        headers: Optional[Dict[str, str]] = None,
    ):
        assert isinstance(content, ExceptionContent)
        body = content.enrich()
        super().__init__(
            status_code=body.status_code,
            detail=body.model_dump(mode="json"),
            headers=headers,
        )


async def general_handler(request: Request, exc: Exception):
    """This is the default exception handler for all exceptions."""
    body = ExceptionContent(message=str(exc), error=ApiError.UNKNOWN_ERROR)
    return JSONResponse(
        status_code=body.error.status_code, content=HTTPException(content=body).detail
    )


async def request_validation_handler(request: Request, exc: RequestValidationError):
    """This is the exception handler for all request validation errors."""
    body = ExceptionContent(message=str(exc), error=ApiError.INVALID_DATA_REQUEST)
    return JSONResponse(
        status_code=body.error.status_code, content=HTTPException(content=body).detail
    )


async def response_validation_handler(request: Request, exc: ResponseValidationError):
    """This is the exception handler for all response validation errors."""
    body = ExceptionContent(message=str(exc), error=ApiError.INVALID_DATA_RESPONSE)
    return JSONResponse(
        status_code=body.error.status_code, content=HTTPException(content=body).detail
    )


async def http_handler(request: Request, exc: HTTPException):
    """This is the exception handler for HTTPExceptions. It unwraps the HTTPException and returns the detail in a flat JSON response."""
    return JSONResponse(status_code=exc.status_code, content=exc.detail)


def register_exception_handlers(app: FastAPI):
    """Utility to register serveral exception handlers in one go. Catches Exception, HTTPException and Data Validation errors and responds with a unified json body."""
    app.add_exception_handler(Exception, general_handler)
    app.add_exception_handler(FastAPIHTTPException, http_handler)
    app.add_exception_handler(RequestValidationError, request_validation_handler)
    app.add_exception_handler(ResponseValidationError, response_validation_handler)


exception_response = {
    "default": {"model": ExceptionDetail, "description": "Error response"}
}



# coding: utf-8

"""
Marketcap Service API

     API for retrieving historical marketcap data, available exchanges, and indicators.          ## Features     - Historical marketcap data     - OHLCV data with marketcap     - Technical indicators (KER, SMA)     - Exchange and symbol mappings     - Error logs

The version of the OpenAPI document: 1.0.0
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import json
from enum import Enum
from typing_extensions import Self


class Market(str, Enum):
    """
    Market
    """

    """
    allowed enum values
    """
    SPOT = "spot"
    FUTURES = "futures"

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of Market from a JSON string"""
        return cls(json.loads(json_str))



from crypticorn.hive import (
    ApiClient,
    Configuration,
    ModelsApi,
    DataApi,
    StatusApi,
    Configuration,
)
from crypticorn.common import apikey_header as aph


class HiveClient:
    """
    A client for interacting with the Crypticorn Hive API.
    """

    config_class = Configuration

    def __init__(
        self,
        config: Configuration,
    ):
        self.config = config
        self.base_client = ApiClient(configuration=self.config)
        # Instantiate all the endpoint clients
        self.models = ModelsApi(self.base_client)
        self.data = DataApi(self.base_client)
        self.status = StatusApi(self.base_client)



# coding: utf-8

"""
Marketcap Service API

     API for retrieving historical marketcap data, available exchanges, and indicators.          ## Features     - Historical marketcap data     - OHLCV data with marketcap     - Technical indicators (KER, SMA)     - Exchange and symbol mappings     - Error logs

The version of the OpenAPI document: 1.0.0
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from pydantic import BaseModel, ConfigDict, StrictBool, StrictStr
from typing import Any, ClassVar, Dict, List, Optional
from typing import Optional, Set
from typing_extensions import Self


class ErrorResponse(BaseModel):
    """
    ErrorResponse
    """  # noqa: E501

    success: Optional[StrictBool] = False
    message: StrictStr
    error_code: StrictStr
    details: Optional[Dict[str, Any]] = None
    timestamp: Optional[datetime] = None
    __properties: ClassVar[List[str]] = [
        "success",
        "message",
        "error_code",
        "details",
        "timestamp",
    ]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of ErrorResponse from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # set to None if details (nullable) is None
        # and model_fields_set contains the field
        if self.details is None and "details" in self.model_fields_set:
            _dict["details"] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of ErrorResponse from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate(
            {
                "success": (
                    obj.get("success") if obj.get("success") is not None else False
                ),
                "message": obj.get("message"),
                "error_code": obj.get("error_code"),
                "details": obj.get("details"),
                "timestamp": obj.get("timestamp"),
            }
        )
        return _obj



# coding: utf-8

"""
Klines Service API

     API for retrieving OHLCV data, funding rates, and symbol information from Binance.          ## WebSocket Support     Connect to `/ws` to receive real-time OHLCV updates.          Example subscription message:     ```json     {         \"action\": \"subscribe\",         \"market\": \"spot\",         \"symbol\": \"BTCUSDT\",         \"timeframe\": \"15m\"     }     ```

The version of the OpenAPI document: 1.0.0
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
from inspect import getfullargspec
import json
import pprint
import re  # noqa: F401
from pydantic import (
    BaseModel,
    ConfigDict,
    Field,
    StrictStr,
    ValidationError,
    field_validator,
)
from typing import Optional
from crypticorn.klines.client.models.history_error_response import HistoryErrorResponse
from crypticorn.klines.client.models.history_no_data_response import (
    HistoryNoDataResponse,
)
from crypticorn.klines.client.models.history_success_response import (
    HistorySuccessResponse,
)
from typing import Union, Any, List, Set, TYPE_CHECKING, Optional, Dict
from typing_extensions import Literal, Self
from pydantic import Field

RESPONSEGETUDFHISTORY_ANY_OF_SCHEMAS = [
    "HistoryErrorResponse",
    "HistoryNoDataResponse",
    "HistorySuccessResponse",
]


class ResponseGetUdfHistory(BaseModel):
    """
    ResponseGetUdfHistory
    """

    # data type: HistorySuccessResponse
    anyof_schema_1_validator: Optional[HistorySuccessResponse] = None
    # data type: HistoryNoDataResponse
    anyof_schema_2_validator: Optional[HistoryNoDataResponse] = None
    # data type: HistoryErrorResponse
    anyof_schema_3_validator: Optional[HistoryErrorResponse] = None
    if TYPE_CHECKING:
        actual_instance: Optional[
            Union[HistoryErrorResponse, HistoryNoDataResponse, HistorySuccessResponse]
        ] = None
    else:
        actual_instance: Any = None
    any_of_schemas: Set[str] = {
        "HistoryErrorResponse",
        "HistoryNoDataResponse",
        "HistorySuccessResponse",
    }

    model_config = {
        "validate_assignment": True,
        "protected_namespaces": (),
    }

    def __init__(self, *args, **kwargs) -> None:
        if args:
            if len(args) > 1:
                raise ValueError(
                    "If a position argument is used, only 1 is allowed to set `actual_instance`"
                )
            if kwargs:
                raise ValueError(
                    "If a position argument is used, keyword arguments cannot be used."
                )
            super().__init__(actual_instance=args[0])
        else:
            super().__init__(**kwargs)

    @field_validator("actual_instance")
    def actual_instance_must_validate_anyof(cls, v):
        instance = ResponseGetUdfHistory.model_construct()
        error_messages = []
        # validate data type: HistorySuccessResponse
        if not isinstance(v, HistorySuccessResponse):
            error_messages.append(
                f"Error! Input type `{type(v)}` is not `HistorySuccessResponse`"
            )
        else:
            return v

        # validate data type: HistoryNoDataResponse
        if not isinstance(v, HistoryNoDataResponse):
            error_messages.append(
                f"Error! Input type `{type(v)}` is not `HistoryNoDataResponse`"
            )
        else:
            return v

        # validate data type: HistoryErrorResponse
        if not isinstance(v, HistoryErrorResponse):
            error_messages.append(
                f"Error! Input type `{type(v)}` is not `HistoryErrorResponse`"
            )
        else:
            return v

        if error_messages:
            # no match
            raise ValueError(
                "No match found when setting the actual_instance in ResponseGetUdfHistory with anyOf schemas: HistoryErrorResponse, HistoryNoDataResponse, HistorySuccessResponse. Details: "
                + ", ".join(error_messages)
            )
        else:
            return v

    @classmethod
    def from_dict(cls, obj: Dict[str, Any]) -> Self:
        return cls.from_json(json.dumps(obj))

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Returns the object represented by the json string"""
        instance = cls.model_construct()
        error_messages = []
        # anyof_schema_1_validator: Optional[HistorySuccessResponse] = None
        try:
            instance.actual_instance = HistorySuccessResponse.from_json(json_str)
            return instance
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # anyof_schema_2_validator: Optional[HistoryNoDataResponse] = None
        try:
            instance.actual_instance = HistoryNoDataResponse.from_json(json_str)
            return instance
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # anyof_schema_3_validator: Optional[HistoryErrorResponse] = None
        try:
            instance.actual_instance = HistoryErrorResponse.from_json(json_str)
            return instance
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))

        if error_messages:
            # no match
            raise ValueError(
                "No match found when deserializing the JSON string into ResponseGetUdfHistory with anyOf schemas: HistoryErrorResponse, HistoryNoDataResponse, HistorySuccessResponse. Details: "
                + ", ".join(error_messages)
            )
        else:
            return instance

    def to_json(self) -> str:
        """Returns the JSON representation of the actual instance"""
        if self.actual_instance is None:
            return "null"

        if hasattr(self.actual_instance, "to_json") and callable(
            self.actual_instance.to_json
        ):
            return self.actual_instance.to_json()
        else:
            return json.dumps(self.actual_instance)

    def to_dict(
        self,
    ) -> Optional[
        Union[
            Dict[str, Any],
            HistoryErrorResponse,
            HistoryNoDataResponse,
            HistorySuccessResponse,
        ]
    ]:
        """Returns the dict representation of the actual instance"""
        if self.actual_instance is None:
            return None

        if hasattr(self.actual_instance, "to_dict") and callable(
            self.actual_instance.to_dict
        ):
            return self.actual_instance.to_dict()
        else:
            return self.actual_instance

    def to_str(self) -> str:
        """Returns the string representation of the actual instance"""
        return pprint.pformat(self.model_dump())



from typing import Any, Union
from decimal import Decimal
import string
import random
from fastapi import HTTPException
from fastapi import status
from typing_extensions import deprecated

from crypticorn.common import ApiError


def throw_if_none(value: Any, message: Union[ApiError, str]) -> None:
    """Throws an FastAPI HTTPException if the value is None."""
    if value is None:
        raise HTTPException(
            status_code=status.HTTP_404_NOT_FOUND,
            detail=message.identifier if isinstance(message, ApiError) else message,
        )


def throw_if_falsy(value: Any, message: Union[ApiError, str]) -> None:
    """Throws an FastAPI HTTPException if the value is False."""
    if not value:
        raise HTTPException(
            status_code=status.HTTP_400_BAD_REQUEST,
            detail=message.identifier if isinstance(message, ApiError) else message,
        )


def gen_random_id(length: int = 20) -> str:
    """Generate a random base62 string (a-zA-Z0-9) of specified length.
    Kucoin max 40, bingx max 40"""
    charset = string.ascii_letters + string.digits
    return "".join(random.choice(charset) for _ in range(length))


@deprecated("Use math.isclose instead. Will be removed in a future version.")
def is_equal(
    a: float | Decimal,
    b: float | Decimal,
    rel_tol: float = 1e-9,
    abs_tol: float = 0.0,
) -> bool:
    """
    Compare two Decimal numbers for approximate equality.
    """
    if not isinstance(a, Decimal):
        a = Decimal(str(a))
    if not isinstance(b, Decimal):
        b = Decimal(str(b))

    # Convert tolerances to Decimal
    return Decimal(abs(a - b)) <= max(
        Decimal(str(rel_tol)) * max(abs(a), abs(b)), Decimal(str(abs_tol))
    )


def optional_import(module_name: str, extra_name: str) -> Any:
    """
    Import a module optionally.
    """
    try:
        return __import__(module_name)
    except ImportError as e:
        extra = f"[{extra_name}]"
        raise ImportError(
            f"Optional dependency '{module_name}' is required for this feature. "
            f"Install it with: pip install crypticorn{extra}"
        ) from e



# coding: utf-8

"""
Payment API

No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

The version of the OpenAPI document: 0.1.0
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import json
from enum import Enum
from typing_extensions import Self


class NowPaymentStatus(str, Enum):
    """
    Payment status for the payment
    """

    """
    allowed enum values
    """
    WAITING = "waiting"
    CONFIRMING = "confirming"
    CONFIRMED = "confirmed"
    SENDING = "sending"
    PARTIALLY_PAID = "partially_paid"
    FINISHED = "finished"
    FAILED = "failed"
    REFUNDED = "refunded"
    EXPIRED = "expired"

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of NowPaymentStatus from a JSON string"""
        return cls(json.loads(json_str))



# coding: utf-8

"""
Trading API

No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

The version of the OpenAPI document: 0.1.0
Generated by OpenAPI Generator (https://openapi-generator.tech)

Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional
from crypticorn.trade.client.models.exchange import Exchange
from typing import Optional, Set
from typing_extensions import Self


class ExchangeKeyModel(BaseModel):
    """
    ExchangeKeyModel
    """  # noqa: E501

    created_at: Optional[StrictInt] = None
    updated_at: Optional[StrictInt] = None
    id: Optional[StrictStr] = None
    exchange: Exchange = Field(description="Exchange name")
    api_key: Optional[StrictStr] = None
    secret: Optional[StrictStr] = None
    passphrase: Optional[StrictStr] = None
    label: StrictStr = Field(description="Label for the API key")
    enabled: Optional[StrictBool] = None
    user_id: Optional[StrictStr] = None
    __properties: ClassVar[List[str]] = [
        "created_at",
        "updated_at",
        "id",
        "exchange",
        "api_key",
        "secret",
        "passphrase",
        "label",
        "enabled",
        "user_id",
    ]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of ExchangeKeyModel from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # set to None if created_at (nullable) is None
        # and model_fields_set contains the field
        if self.created_at is None and "created_at" in self.model_fields_set:
            _dict["created_at"] = None

        # set to None if updated_at (nullable) is None
        # and model_fields_set contains the field
        if self.updated_at is None and "updated_at" in self.model_fields_set:
            _dict["updated_at"] = None

        # set to None if id (nullable) is None
        # and model_fields_set contains the field
        if self.id is None and "id" in self.model_fields_set:
            _dict["id"] = None

        # set to None if api_key (nullable) is None
        # and model_fields_set contains the field
        if self.api_key is None and "api_key" in self.model_fields_set:
            _dict["api_key"] = None

        # set to None if secret (nullable) is None
        # and model_fields_set contains the field
        if self.secret is None and "secret" in self.model_fields_set:
            _dict["secret"] = None

        # set to None if passphrase (nullable) is None
        # and model_fields_set contains the field
        if self.passphrase is None and "passphrase" in self.model_fields_set:
            _dict["passphrase"] = None

        # set to None if enabled (nullable) is None
        # and model_fields_set contains the field
        if self.enabled is None and "enabled" in self.model_fields_set:
            _dict["enabled"] = None

        # set to None if user_id (nullable) is None
        # and model_fields_set contains the field
        if self.user_id is None and "user_id" in self.model_fields_set:
            _dict["user_id"] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of ExchangeKeyModel from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate(
            {
                "created_at": obj.get("created_at"),
                "updated_at": obj.get("updated_at"),
                "id": obj.get("id"),
                "exchange": obj.get("exchange"),
                "api_key": obj.get("api_key"),
                "secret": obj.get("secret"),
                "passphrase": obj.get("passphrase"),
                "label": obj.get("label"),
                "enabled": obj.get("enabled"),
                "user_id": obj.get("user_id"),
            }
        )
        return _obj
